@@ -13,7 +13,7 @@
 #include	"comfax.h"
 #include	"rfc822/rfc822.h"
 #include	"waitlib/waitlib.h"
-#include	"unicode/unicode.h"
+#include	<courier-unicode.h>
 #include	"numlib/numlib.h"
 #include	<stdlib.h>
 #include	<locale.h>
@@ -466,7 +466,7 @@ static void fax(struct moduledel *p)
 			size_t npages=0;
 			char fmtbuf[NUMBUFSIZE];
 			char fmtbuf1[NUMBUFSIZE];
-			char fmtbuf2[NUMBUFSIZE+100];
+			char fmtbuf2[NUMBUFSIZE*2+100];
 
 			for (pp=page_list; pp; pp=pp->next)
 				++npages;
@@ -631,4 +631,3 @@ static int faxsend_cleanup(int errcode, char *errmsg, void *vp)
 			      COMCTLFILE_DELFAIL_NOTRACK), 0);
 	return (-1);
 }
-
