@@ -1,8 +1,6 @@
-#! perl
 #
 # TITLE: Password authentication modules
 #
-# $Id: admin-10password.pl,v 1.5 2003/07/06 16:16:37 mrsam Exp $
 #
 # Copyright 2001 Double Precision, Inc.  See COPYING for
 # distribution information.
@@ -34,7 +32,7 @@ if ($cgi->param("Up") && $authmodule_param)
 
 	    ReplaceEnvVarConfigFile("authdaemonrc", "authmodulelist",
 				    join(" ", @mods));
-	    changed("$libexecdir/authlib/authdaemond restart");
+	    changed("$authdaemond restart");
 	    $selected= $i-1;
 	    last;
 	}
@@ -56,7 +54,7 @@ if ($cgi->param("Down") && $authmodule_param)
 
 	    ReplaceEnvVarConfigFile("authdaemonrc", "authmodulelist",
 				    join(" ", @mods));
-	    changed("$libexecdir/authlib/authdaemond restart");
+	    changed("$authdaemond restart");
 	    $selected=$i+1;
 	    last;
 	}
@@ -74,7 +72,7 @@ if ($cgi->param("Delete") && $authmodule_param)
 	splice @mods, $i, 1;
 	ReplaceEnvVarConfigFile("authdaemonrc", "authmodulelist",
 				join(" ", @mods));
-	changed("$libexecdir/authlib/authdaemond restart");
+	changed("$authdaemond restart");
 	last;
     }
 }
@@ -84,7 +82,7 @@ if ($authmodule_param=$cgi->param("authmodulelistorig"))
     push @mods, $authmodule_param;
     ReplaceEnvVarConfigFile("authdaemonrc", "authmodulelist",
 			    join(" ", @mods));
-    changed("$libexecdir/authlib/authdaemond restart");
+    changed("$authdaemond restart");
     $errstr="\@SAVED\@";
 }
 
@@ -109,30 +107,6 @@ foreach (grep(/./, split (/\s+/, $authmodulelistorig)))
 }
 $authmodulelistorig_current .= "</select></tt>";
 
-my $authdaemond=$$vars{'version'};
-
-my $new_authdaemond=$cgi->param("authdaemond");
-
-if ($new_authdaemond)
-{
-    my $good=0;
-
-    ($good=1, $new_authdaemond="") if $new_authdaemond eq "(default)";
-    ($good=1, $new_authdaemond=$1) if $new_authdaemond =~ /([a-z0-9\.]*)/;
-
-    if ($good)
-    {
-	if ($new_authdaemond eq "" ||
-	    -f "$libexecdir/authlib/$new_authdaemond")
-	{
-	    ReplaceEnvVarConfigFile("authdaemonrc", "version",
-				    $new_authdaemond);
-	    $authdaemond=$new_authdaemond;
-	    changed("$libexecdir/authlib/authdaemond stop ; $libexecdir/authlib/authdaemond start");
-	}
-    }
-}
-
 my $daemons=$$vars{'daemons'};
 
 my $daemons_params=$cgi->param("daemons");
@@ -145,28 +119,10 @@ if ($daemons_params =~ /([1-9][0-9]*)/)
     $daemons=$daemons_params;
 }
 
-my $authdaemond_html="<tt><select name=authdaemond>";
-
-foreach (("(default)",
-	  "authdaemond.plain", "authdaemond.mysql",
-	  "authdaemond.pgsql", "authdaemond.ldap"))
-{
-    my $n=$_;
-
-    $authdaemond_html .= "<option value=\"$n\" "
-	. ($n eq $authdaemond ? "selected=\"selected\"":"") . ">";
-
-    $authdaemond_html .= $n;
-}
-
-$authdaemond_html .= "</select></tt>";
-
-
 display_form("admin-10password.html",
 	     {
 		 "AUTHMODULELIST" => $authmodulelist_current,
 		 "AUTHMODULELISTORIG" => $authmodulelistorig_current,
-		 "AUTHDAEMOND" => $authdaemond_html,
 		 "DAEMONS" => $daemons
 	     }
 	     );
