@@ -1,5 +1,5 @@
 /*
-** Copyright 2000-2009 Double Precision, Inc.
+** Copyright 2000-2016 Double Precision, Inc.
 ** See COPYING for distribution information.
 */
 #include	"config.h"
@@ -55,6 +55,31 @@
 
 #include	<sys/time.h>
 
+struct proto_ops {
+    char *n;
+    const SSL_METHOD * (*m)();
+    int o;
+};
+struct proto_ops op_list[] =
+{
+#ifdef SSL_OP_NO_TLSv1
+#ifdef SSL_OP_NO_TLSv1_1
+    { "TLSv1.2+",  &SSLv23_method,  SSL_OP_ALL|SSL_OP_NO_SSLv2|SSL_OP_NO_SSLv3|SSL_OP_NO_TLSv1|SSL_OP_NO_TLSv1_1 },
+    { "TLSv1.2",   &SSLv23_method,  SSL_OP_ALL|SSL_OP_NO_SSLv2|SSL_OP_NO_SSLv3|SSL_OP_NO_TLSv1|SSL_OP_NO_TLSv1_1 },
+#endif
+#endif
+
+#ifdef SSL_OP_NO_TLSv1
+    { "TLSv1.1+",  &SSLv23_method,  SSL_OP_ALL|SSL_OP_NO_SSLv2|SSL_OP_NO_SSLv3|SSL_OP_NO_TLSv1 },
+    { "TLSv1.1",   &SSLv23_method,  SSL_OP_ALL|SSL_OP_NO_SSLv2|SSL_OP_NO_SSLv3|SSL_OP_NO_TLSv1 },
+#endif
+    { "TLSv1",     &SSLv23_method,  SSL_OP_ALL|SSL_OP_NO_SSLv2|SSL_OP_NO_SSLv3 },
+    { "TLS1",      &SSLv23_method,  SSL_OP_ALL|SSL_OP_NO_SSLv2|SSL_OP_NO_SSLv3 },
+    { "",          &SSLv23_method,  SSL_OP_ALL|SSL_OP_NO_SSLv2|SSL_OP_NO_SSLv3 },
+    { NULL,        &SSLv23_method,  SSL_OP_ALL|SSL_OP_NO_SSLv2|SSL_OP_NO_SSLv3 },
+};
+
+
 /***** TODO *****/
 
 /* #define TLSCACHEDEBUG */
@@ -138,16 +163,16 @@ static int verifypeer(const struct tls_info *info, SSL *ssl)
 	{
 		STACK_OF(X509) *peer_cert_chain=SSL_get_peer_cert_chain(ssl);
 
-		if (peer_cert_chain && peer_cert_chain->stack.num > 0)
+		if (peer_cert_chain && sk_X509_num(peer_cert_chain) > 0)
 		{
-			X509 *xx=(X509 *)peer_cert_chain->stack.data[0];
+			X509 *xx=sk_X509_value(peer_cert_chain, 0);
 
 			if (xx)
 				subj=X509_get_subject_name(xx);
 		}
 	}
 
-	
+
 	nentries=0;
 	if (subj)
 		nentries=X509_NAME_entry_count(subj);
@@ -161,7 +186,7 @@ static int verifypeer(const struct tls_info *info, SSL *ssl)
 		ASN1_STRING *d;
 
 		int dlen;
-		unsigned char *ddata;
+		const unsigned char *ddata;
 
 		e=X509_NAME_get_entry(subj, j);
 		if (!e)
@@ -176,8 +201,11 @@ static int verifypeer(const struct tls_info *info, SSL *ssl)
 		obj_name=OBJ_nid2sn(OBJ_obj2nid(o));
 
 		dlen=ASN1_STRING_length(d);
+#ifdef HAVE_OPENSSL110
+		ddata=ASN1_STRING_get0_data(d);
+#else
 		ddata=ASN1_STRING_data(d);
-
+#endif
 		if (strcasecmp(obj_name, "CN") == 0)
 		{
 			if (dlen >= sizeof(domain)-1)
@@ -280,17 +308,107 @@ static void load_dh_params(SSL_CTX *ctx, const char *filename,
 			DH_free(dh);
 		}
 		else
-			sslerror(info, filename, -1);
+		{
+			/*
+			** If the certificate file does not have DH parameters,
+			** swallow the error.
+			*/
+
+			int err=ERR_peek_last_error();
+
+			if (ERR_GET_LIB(err) == ERR_LIB_PEM
+			    && ERR_GET_REASON(err) == PEM_R_NO_START_LINE)
+			{
+				ERR_clear_error();
+			}
+			else
+			{
+				sslerror(info, filename, -1);
+			}
+		}
 		BIO_free(bio);
 	}
 	else
 		sslerror(info, filename, -1);
 }
 
+static int check_readable_file(const char *filename)
+{
+	return (access(filename, R_OK) == 0) ? 1 : 0;
+}
+
+#ifdef HAVE_OPENSSL_SNI
+static char *get_servername_concated_readable_file(const char *filename,
+				   const char *servername,
+				   struct tls_info *info)
+{
+	char *filename_buffer;
+	char *p;
+
+	if (!filename || !servername) return NULL;
+	if (!*filename || !*servername) return NULL;
+
+	filename_buffer=malloc(strlen(filename)+strlen(servername)+2);
+	if (!filename_buffer)
+	{
+		nonsslerror(info, "malloc");
+		exit(1);
+	}
+
+	strcat(strcpy(filename_buffer, filename), ".");
+
+	p=filename_buffer + strlen(filename_buffer);
+
+	while ((*p=*servername) != 0)
+	{
+		if (*p == '/')
+			*p='.'; /* Script kiddie check */
+		++p;
+		++servername;
+	}
+	if (check_readable_file(filename_buffer)) {
+		return filename_buffer;
+	}
+
+	free(filename_buffer);
+	return NULL;
+}
+#endif
+
+static char *get_ip_concated_readable_file(SSL_CTX *ctx, const char *filename, const char *ip)
+{
+	char *test_file;
+	const struct tls_info *info=SSL_CTX_get_app_data(ctx);
+
+	if (!filename || !ip) return NULL;
+	if (!*filename || !*ip) return NULL;
+
+	test_file= malloc(strlen(filename)+strlen(ip)+2);
+	if (!test_file)
+	{
+		nonsslerror(info, "malloc");
+		exit(1);
+	}
+
+	strcpy(test_file, filename);
+	strcat(test_file, ".");
+	strcat(test_file, ip);
+
+	if (check_readable_file(test_file)) {
+		return test_file;
+	}
+
+	free(test_file);
+	return NULL;
+}
+
 static int read_certfile(SSL_CTX *ctx, const char *filename,
+			      const char *private_key_filename,
 			      int *cert_file_flags)
 {
 	const struct tls_info *info=SSL_CTX_get_app_data(ctx);
+	const char *privatekey_file = (private_key_filename == NULL)
+				    ? filename : private_key_filename;
 
 	if(!SSL_CTX_use_certificate_chain_file(ctx, filename))
 	{
@@ -300,7 +418,7 @@ static int read_certfile(SSL_CTX *ctx, const char *filename,
 
 	load_dh_params(ctx, filename, cert_file_flags);
 
-	if(!SSL_CTX_use_PrivateKey_file(ctx, filename, SSL_FILETYPE_PEM))
+	if(!SSL_CTX_use_PrivateKey_file(ctx, privatekey_file, SSL_FILETYPE_PEM))
 	{
 		sslerror(info, filename, -1);
 		return (0);
@@ -308,36 +426,39 @@ static int read_certfile(SSL_CTX *ctx, const char *filename,
 	return (1);
 }
 
-static int process_certfile(SSL_CTX *ctx, const char *certfile, const char *ip,
+static int process_certfile(SSL_CTX *ctx, const char *certfile,
+			    const char *private_key_file,
+			    const char *ip,
 			    int (*func)(SSL_CTX *, const char *,
+					const char *,
 					int *),
 			    int *cert_file_flags)
 {
 	if (ip && *ip)
 	{
 		char *test_file;
+		char *test_private_key_file;
 
 		if (strncmp(ip, "::ffff:", 7) == 0 && strchr(ip, '.'))
-			return (process_certfile(ctx, certfile, ip+7, func, cert_file_flags));
+			return (process_certfile(ctx, certfile, private_key_file, ip+7, func, cert_file_flags));
 
-		test_file= malloc(strlen(certfile)+strlen(ip)+2);
-
-		strcpy(test_file, certfile);
-		strcat(test_file, ".");
-		strcat(test_file, ip);
-
-		if (access(test_file, R_OK) == 0)
+		test_file= get_ip_concated_readable_file(ctx, certfile, ip);
+		test_private_key_file = get_ip_concated_readable_file(ctx, private_key_file, ip);
+		if (test_file != NULL)
 		{
-			int rc= (*func)(ctx, test_file,
+			int rc= (*func)(ctx, test_file, test_private_key_file,
 					cert_file_flags);
 
 			free(test_file);
+			if (test_private_key_file) free(test_private_key_file);
+
 			return rc;
 		}
-		free(test_file);
+		if (test_private_key_file) free(test_private_key_file);
 	}
 
-	return (*func)(ctx, certfile, cert_file_flags);
+	private_key_file = check_readable_file(private_key_file) ? private_key_file : NULL;
+	return (*func)(ctx, certfile, private_key_file, cert_file_flags);
 }
 
 static int client_cert_cb(ssl_handle ssl, X509 **x509, EVP_PKEY **pkey)
@@ -386,16 +507,15 @@ static int client_cert_cb(ssl_handle ssl, X509 **x509, EVP_PKEY **pkey)
 			continue;
 		}
 
-		for (i=0; client_cas && i<client_cas->stack.num; i++)
+		for (i=0; client_cas && i<sk_X509_NAME_num(client_cas); i++)
 		{
-			X509_NAME *cert=(X509_NAME *)client_cas->stack.data[i];
+			X509_NAME *cert=sk_X509_NAME_value(client_cas, i);
 
-			if (X509_NAME_cmp(cert,
-					  x->cert_info->issuer) == 0)
+			if (X509_NAME_cmp(cert, X509_get_issuer_name(x)) == 0)
 				break;
 		}
 
-		if (!client_cas || i >= client_cas->stack.num)
+		if (!client_cas || i >= sk_X509_NAME_num(client_cas))
 		{
 			BIO_free(certbio);
 			continue;
@@ -448,7 +568,63 @@ static int client_cert_cb(ssl_handle ssl, X509 **x509, EVP_PKEY **pkey)
 	return rc;
 }
 
+static SSL_CTX *tls_create_int(int isserver, const struct tls_info *info,
+			       int internal);
+
+static int server_cert_cb(ssl_handle ssl, int *ad, void *arg)
+{
+#ifdef HAVE_OPENSSL_SNI
+	struct tls_info *info=(struct tls_info *)SSL_get_app_data(ssl);
+	const char *servername=SSL_get_servername(ssl,
+						  TLSEXT_NAMETYPE_host_name);
+	const char *certfile=safe_getenv(info, "TLS_CERTFILE");
+	const char *private_keyfile=safe_getenv(info, "TLS_PRIVATE_KEYFILE");
+	int cert_file_flags=0;
+	char *cert_file_buffer;
+	char *private_keyfile_buffer;
+
+	if (!servername || !certfile)
+		return SSL_TLSEXT_ERR_OK;
+
+	cert_file_buffer = get_servername_concated_readable_file(certfile, servername, info);
+	private_keyfile_buffer = get_servername_concated_readable_file(private_keyfile, servername, info);
+	if (cert_file_buffer != NULL)
+	{
+		SSL_CTX *orig_ctx=SSL_get_SSL_CTX(ssl);
+		SSL_CTX *temp_ctx=tls_create_int(1, info, 1);
+		int rc;
+
+		if (!temp_ctx)
+		{
+			(*info->tls_err_msg)("Cannot load certificate file",
+					     info->app_data);
+			exit(1);
+		}
+		SSL_set_SSL_CTX(ssl, temp_ctx);
+		rc=read_certfile(orig_ctx, cert_file_buffer, private_keyfile_buffer, &cert_file_flags);
+		SSL_set_SSL_CTX(ssl, orig_ctx);
+		tls_destroy(temp_ctx);
+		if (!rc)
+		{
+			(*info->tls_err_msg)("Cannot load certificate file",
+					     info->app_data);
+			exit(1);
+		}
+	}
+	if (cert_file_buffer) free(cert_file_buffer);
+	if (private_keyfile_buffer) free(private_keyfile_buffer);
+
+#endif
+	return SSL_TLSEXT_ERR_OK;
+}
+
 SSL_CTX *tls_create(int isserver, const struct tls_info *info)
+{
+	return tls_create_int(isserver, info, 0);
+}
+
+SSL_CTX *tls_create_int(int isserver, const struct tls_info *info,
+			int internal)
 {
 	SSL_CTX *ctx;
 	const char *protocol=safe_getenv(info, "TLS_PROTOCOL");
@@ -456,6 +632,7 @@ SSL_CTX *tls_create(int isserver, const struct tls_info *info)
 	int session_timeout=atoi(safe_getenv(info, "TLS_TIMEOUT"));
 	const char *dhparamsfile=safe_getenv(info, "TLS_DHPARAMS");
 	const char *certfile=safe_getenv(info, "TLS_CERTFILE");
+	const char *private_keyfile=safe_getenv(info, "TLS_PRIVATE_KEYFILE");
 	const char *s;
 	struct stat stat_buf;
 	const char *peer_cert_dir=NULL;
@@ -465,6 +642,7 @@ SSL_CTX *tls_create(int isserver, const struct tls_info *info)
 	const SSL_METHOD *method=NULL;
 	long options;
 	int cert_file_flags;
+	struct proto_ops *opp;
 
 	if (!*ssl_cipher_list)
 		ssl_cipher_list=NULL;
@@ -472,6 +650,9 @@ SSL_CTX *tls_create(int isserver, const struct tls_info *info)
 	if (!*certfile)
 		certfile=NULL;
 
+	if (!*private_keyfile)
+		private_keyfile=NULL;
+
 	if (!*dhparamsfile)
 		dhparamsfile=NULL;
 
@@ -496,8 +677,10 @@ SSL_CTX *tls_create(int isserver, const struct tls_info *info)
 		if (first)
 		{
 			first=0;
+#if OPENSSL_API_COMPAT < 0x10100000L
 			SSL_load_error_strings();
 			SSLeay_add_ssl_algorithms();
+#endif
 
 			while (RAND_status() != 1)
 			{
@@ -522,31 +705,13 @@ SSL_CTX *tls_create(int isserver, const struct tls_info *info)
 	info_copy->isserver=isserver;
 	info_copy->certificate_verified=0;
 
-	options=SSL_OP_ALL;
-
-	method=((!protocol || !*protocol)
-		? NULL:
-		strcmp(protocol, "SSL3") == 0
-			? SSLv3_method():
-		strcmp(protocol, "SSL23") == 0
-			? SSLv23_method():
-		strcmp(protocol, "TLSv1") == 0
-		? TLSv1_method():
-#ifdef HAVE_TLSV1_1_METHOD
-		strcmp(protocol, "TLSv1.1") == 0
-		? TLSv1_1_method():
-#endif
-#ifdef HAVE_TLSV1_2_METHOD
-		strcmp(protocol, "TLSv1.2") == 0
-		? TLSv1_2_method():
-#endif
-		NULL);
-
-	if (!method)
+	for (opp=&op_list[0];opp->n!=NULL;opp++)
 	{
-		method=SSLv23_method();
-		options|=SSL_OP_NO_SSLv2;
-	}
+	    if (strcmp(opp->n,protocol)==0)
+		break;
+	};
+	options=opp->o;
+	method=opp->m();
 
 	ctx=SSL_CTX_new(method);
 
@@ -560,14 +725,35 @@ SSL_CTX *tls_create(int isserver, const struct tls_info *info)
 	SSL_CTX_set_options(ctx, options);
 
 	if (!ssl_cipher_list)
-		ssl_cipher_list="SSLv3:TLSv1:HIGH:!LOW:!MEDIUM:!EXP:!NULL:!aNULL@STRENGTH";
+		ssl_cipher_list="TLSv1:HIGH:!LOW:!MEDIUM:!EXP:!NULL:!aNULL@STRENGTH";
 
 	SSL_CTX_set_cipher_list(ctx, ssl_cipher_list);
 	SSL_CTX_set_timeout(ctx, session_timeout);
 
-	info_copy->tlscache=NULL;
-	init_session_cache(info_copy, ctx);
+#ifndef OPENSSL_NO_ECDH
+#ifdef SSL_CTX_set_ecdh_auto
+	SSL_CTX_set_ecdh_auto(ctx, 1);
+#else
+#ifdef NID_X9_62_prime256v1
+	{
+		EC_KEY *key = EC_KEY_new_by_curve_name(NID_X9_62_prime256v1);
 
+		SSL_CTX_set_tmp_ecdh(ctx, key);
+		EC_KEY_free(key);
+	}
+#endif
+#endif
+#endif
+	info_copy->tlscache=NULL;
+
+	if (internal)
+	{
+		SSL_CTX_set_session_cache_mode(ctx, SSL_SESS_CACHE_OFF);
+	}
+	else
+	{
+		init_session_cache(info_copy, ctx);
+	}
 
 	s = safe_getenv(info, "TCPLOCALIP");
 
@@ -576,7 +762,7 @@ SSL_CTX *tls_create(int isserver, const struct tls_info *info)
 	if (dhparamsfile)
 		load_dh_params(ctx, dhparamsfile, &cert_file_flags);
 
-	if (certfile && !process_certfile(ctx, certfile, s,
+	if (certfile && !process_certfile(ctx, certfile, private_keyfile, s,
 					  read_certfile,
 					  &cert_file_flags))
 	{
@@ -584,8 +770,6 @@ SSL_CTX *tls_create(int isserver, const struct tls_info *info)
 		return (NULL);
 	}
 
-	SSL_CTX_set_session_cache_mode(ctx, SSL_SESS_CACHE_BOTH);
-
 	n=atoi(safe_getenv(info, "TLS_INTCACHESIZE"));
 
 	if (n > 0)
@@ -664,10 +848,18 @@ SSL_CTX *tls_create(int isserver, const struct tls_info *info)
 				closedir(dirp);
                 }
 	}
+
 	SSL_CTX_set_verify(ctx, get_peer_verify_level(info),
 			   ssl_verify_callback);
-	if (!isserver)
+
+	if (isserver)
+	{
+		SSL_CTX_set_tlsext_servername_callback(ctx, server_cert_cb);
+	}
+	else
+	{
 		SSL_CTX_set_client_cert_cb(ctx, client_cert_cb);
+	}
 	return (ctx);
 }
 
@@ -689,8 +881,14 @@ void tls_destroy(SSL_CTX *ctx)
 
 static int cache_add(SSL *ssl, SSL_SESSION *sess);
 
+#ifdef HAVE_OPENSSL110
+static SSL_SESSION *cache_get(SSL *ssl, const unsigned char *id, int id_len,
+			      int *copyflag);
+#else
 static SSL_SESSION *cache_get(SSL *ssl, unsigned char *id, int id_len,
 			      int *copyflag);
+#endif
+
 static void cache_del(SSL_CTX *ctx, SSL_SESSION *ssl);
 
 static void init_session_cache(struct tls_info *info, SSL_CTX *ctx)
@@ -732,10 +930,12 @@ static int cache_add(SSL *ssl, SSL_SESSION *sess)
 	unsigned char *ucp;
 	time_t timeout= (time_t)SSL_SESSION_get_time(sess)
 		+ SSL_SESSION_get_timeout(sess);
-	void *session_id=(void *)sess->session_id;
-	size_t session_id_len=sess->session_id_length;
+	unsigned int session_id_len;
+	void *session_id;
 	size_t sess_len=i2d_SSL_SESSION(sess, NULL);
 
+	session_id=(void *)SSL_SESSION_get_id(sess, &session_id_len);
+
 	if (sizeof(timeout) + sizeof(session_id_len) + session_id_len +
 	    sess_len > sizeof(buffer))
 	{
@@ -765,7 +965,7 @@ static int cache_add(SSL *ssl, SSL_SESSION *sess)
 }
 
 struct walk_info {
-	unsigned char *id;
+	const unsigned char *id;
 	int id_len;
 	int *copyflag;
 	SSL_SESSION *ret;
@@ -775,8 +975,13 @@ struct walk_info {
 static int get_func(void *rec, size_t recsize,
 		    int *doupdate, void *arg);
 
+#ifdef HAVE_OPENSSL110
+static SSL_SESSION *cache_get(SSL *ssl, const unsigned char *id, int id_len,
+			      int *copyflag)
+#else
 static SSL_SESSION *cache_get(SSL *ssl, unsigned char *id, int id_len,
 			      int *copyflag)
+#endif
 {
 	const struct tls_info *info=SSL_get_app_data(ssl);
 	struct walk_info wi;
@@ -806,7 +1011,7 @@ static int get_func(void *rec, size_t recsize,
 	time_t timeout;
 	size_t session_id_len;
 
-	unsigned char *sess;
+	const unsigned char *sess;
 
 	if (recsize < sizeof(timeout)+sizeof(session_id_len))
 		return (0);
@@ -841,11 +1046,14 @@ static void cache_del(SSL_CTX *ctx, SSL_SESSION *sess)
 {
 	const struct tls_info *info=SSL_CTX_get_app_data(ctx);
 	struct walk_info wi;
+	unsigned int session_id_len;
+	void *session_id;
 
 	wi.now=0;
 
-	wi.id=(unsigned char *)sess->session_id;
-	wi.id_len=sess->session_id_length;
+	session_id=(void *)SSL_SESSION_get_id(sess, &session_id_len);
+	wi.id=(unsigned char *)session_id;
+	wi.id_len=session_id_len;
 	if (tls_cache_walk(info->tlscache, del_func, &wi) < 0)
 		perror("ALERT: tls_cache_walk: ");
 }
@@ -972,6 +1180,13 @@ SSL *tls_connect(SSL_CTX *ctx, int fd)
 	{
 		SSL_set_connect_state(ssl);
 
+#ifdef HAVE_OPENSSL_SNI
+		if (info->peer_verify_domain)
+		{
+			SSL_set_tlsext_host_name(ssl, info->peer_verify_domain);
+		}
+#endif
+
 		if ((rc=SSL_connect(ssl)) > 0)
 		{
 			if (!verifypeer(info, ssl))
@@ -1009,7 +1224,10 @@ void tls_disconnect(SSL *ssl, int fd)
 	fcntl(fd, F_SETFL, 0);
 	SSL_set_shutdown(ssl, SSL_SENT_SHUTDOWN|SSL_RECEIVED_SHUTDOWN);
 	SSL_free(ssl);
+#ifdef HAVE_OPENSSL110
+#else
 	ERR_remove_state(0);
+#endif
 }
 
 /* --------------------------------------- */
@@ -1274,7 +1492,7 @@ static void dump_x509(X509 *x509,
 		ASN1_STRING *d;
 
 		int dlen;
-		unsigned char *ddata;
+		const unsigned char *ddata;
 
 		e=X509_NAME_get_entry(subj, j);
 		if (!e)
@@ -1289,14 +1507,18 @@ static void dump_x509(X509 *x509,
 		obj_name=OBJ_nid2sn(OBJ_obj2nid(o));
 
 		dlen=ASN1_STRING_length(d);
+#ifdef HAVE_OPENSSL110
+		ddata=ASN1_STRING_get0_data(d);
+#else
 		ddata=ASN1_STRING_data(d);
-	
+#endif
+
 		(*dump_func)("   ", -1, dump_arg);
 		(*dump_func)(obj_name, -1, dump_arg);
 		(*dump_func)("=", 1, dump_arg);
 		(*dump_func)((const char *)ddata, dlen, dump_arg);
 		(*dump_func)("\n", 1, dump_arg);
-		
+
 	}
 	(*dump_func)("\n", 1, dump_arg);
 
@@ -1352,8 +1574,9 @@ void tls_dump_connection_info(ssl_handle ssl,
 			}
 		}
 
-		for (i=0; peer_cert_chain && i<peer_cert_chain->stack.num; i++)
-			dump_x509((X509 *)peer_cert_chain->stack.data[i],
+		for (i=0; peer_cert_chain && i<sk_X509_num(peer_cert_chain);
+		     i++)
+			dump_x509((X509 *)sk_X509_value(peer_cert_chain, i),
 				  dump_func, dump_arg);
 	}
 
@@ -1531,7 +1754,8 @@ char *tls_cert_name(const char *buf, size_t buf_size)
 
 	if (x)
 	{
-		p=X509_NAME_oneline(x->cert_info->subject, NULL, 0);
+		X509_get_subject_name(x);
+		p=X509_NAME_oneline(X509_get_subject_name(x), NULL, 0);
 		X509_free(x);
 	}
 	ERR_clear_error();
